@@ -1,11 +1,9 @@
+import math
 import re
-from math import cos, radians
-from os import path
 
-from numpy import NaN, array, interp, argmax, ndarray, min, max, append
-from pandas import isna, notna
+import numpy as np
 
-from .helpers import sqrt, channel_end, Circle
+from .helpers import channel_end, Circle
 from .shape_generator import CrossSection
 
 
@@ -18,14 +16,16 @@ class CrossSectionHolding(CrossSection):
         add_dn (bool): add the channel diameter (DN ...) to the label and output filename
     """
 
-    def __init__(self, label, add_dim=False, add_dn=None, **kwargs):
-        """Initialise the cross section class
+    def __init__(self, label, description=None, add_dim=False, add_dn=None, **kwargs):
+        """Initialise the cross-section class
 
         Args:
-            label (str): name/label/number of the cross section
+            label (str): name/label/number of the cross-section
+            description (Optional[str]): optional description of the cross-section
             add_dim (bool): if the dimensions should be added to :py:attr:`~out_filename` used for the export
             add_dn (Optional[float]): if the channel dimension should be added to :py:attr:`~out_filename`
                                     used for the export enter the diameter as float
+
             **kwargs (object): see :py:attr:`~__init__`
 
         Keyword Args:
@@ -36,53 +36,52 @@ class CrossSectionHolding(CrossSection):
             unit (Optional[str]): enter unit to add the unit in the plots
 
         """
-        if isinstance(label, (float, int)):
-            label = '{:0.0f}'.format(label)
+        if isinstance(label, float):
+            label = f'{label:0.0f}'
         else:
-            label = label
+            label = str(label)
 
-        if not label.startswith('Pr_'):
-            label = 'Pr_' + label
+        if label.startswith('Pr_'):
+            label = label[3:]
 
         self.add_dim = add_dim
         self.add_dn = add_dn
 
+        self.description = description
+
         CrossSection.__init__(self, label, **kwargs)
 
-    def __str__(self):
-        s = CrossSection.__str__(self)
-        if self.add_dim:
-            s += '  |  {:0.0f}'.format(self.height)
-            if self.width:
-                s += 'x{:0.0f}'.format(self.width)
-
-        if self.add_dn:
-            s += '  |  DN{:0.0f}'.format(self.add_dn)
-        return s
+    def __repr__(self):
+        return f'CrossSectionHolding({self})'
 
     @property
-    def out_filename(self):
-        """
-        filename of the figure/text-file to be created
+    def identifier(self):
+        s = 'Pr_' + self.label
 
-        Returns:
-            str: filename
-
-        """
-        file = path.join(self.working_directory, '{}'.format(self.label))
         if self.add_dim:
-            file += '_{:0.0f}'.format(self.height)
+            s += f'_{self.height:0.0f}'  # besser mit "+" aber scho so im Modell ...
             if self.width:
-                file += 'x{:0.0f}'.format(self.width)
+                s += f'x{self.width:0.0f}'
 
         if self.add_dn:
-            file += '_DN{:0.0f}'.format(self.add_dn)
-        return file
+            s += f'+DN{self.add_dn:0.0f}'
+
+        return s
+
+    def name_string(self):
+        return self.identifier.replace('+', ' | ')
+
+    def title_string(self):
+        s = CrossSection.title_string(self)
+        if (self.description is not None) and (self.label != self.description):
+            s += f': {str(self.description).strip()}'
+        return s
 
     ####################################################################################################################
     @classmethod
-    def standard(cls, label, description, height, width=NaN, r_channel=NaN, r_roof=NaN, r_wall=NaN, slope_bench=NaN,
-                 r_round=NaN, r_wall_bottom=NaN, h_bench=NaN, pre_bench=NaN, w_channel=NaN, **kwargs):
+    def standard(cls, label, description=None, height=np.NaN, width=None, r_channel=None, r_roof=None, r_wall=None,
+                 slope_bench=None, r_round=None, r_wall_bottom=None, h_bench=None, pre_bench=None, w_channel=None,
+                 **kwargs):
         """
         standard cross section
 
@@ -142,63 +141,62 @@ class CrossSectionHolding(CrossSection):
         """
 
         # ------------------------------------------------
-        cross_section = cls(label=label, description=description, height=height,
-                            width=(width if notna(width) else None), **kwargs)
+        cross_section = cls(label=label, description=description, height=height, width=width, **kwargs)
 
         # ------------------------------------------------
         # TW-Rinne
-        if notna(r_channel):
+        if r_channel is not None:
             cross_section.add(Circle(r_channel, x_m=r_channel))
 
             # ------------------------------------------------
-            if notna(pre_bench):
+            if pre_bench is not None:
                 cross_section.add(channel_end(r_channel, pre_bench))
 
-                if notna(h_bench) or isna(slope_bench):
+                if (h_bench is not None) or (slope_bench is None):
                     cross_section.add(pre_bench, '°slope')
 
-                if notna(h_bench):
+                if h_bench is not None:
                     cross_section.add(h_bench)
 
-            elif notna(w_channel):
+            elif w_channel is not None:
                 cross_section.add(None, w_channel)
 
             else:
-                if notna(h_bench):
+                if h_bench is not None:
                     cross_section.add(h_bench)
                 else:
                     cross_section.add(r_channel)
-                    if isna(r_round):
+                    if r_round is None:
                         r_round = 0
                     cross_section.add(r_channel + r_round, r_channel)
 
         # ------------------------------------------------
-        if notna(slope_bench):
+        if slope_bench is not None:
             # Berme winkel in °
             cross_section.add(slope_bench, '°slope')
 
         # ------------------------------------------------
-        if isna(r_channel) and isna(slope_bench):
+        if (r_channel is None) and (slope_bench is None):
             cross_section.add(0, width / 2)
 
         # ------------------------------------------------
-        if isna(r_roof):
+        if r_roof is None:
             # eckige Decke
             cross_section.add(None, width / 2)
             cross_section.add(height, width / 2)
 
         else:
-            if isna(r_wall):
+            if r_wall is None:
                 cross_section.add(None, width / 2)
                 cross_section.add(height - r_roof, width / 2)
             else:
                 # ------------------------------------------------
-                h1 = sqrt((r_wall - r_roof) ** 2 - (r_wall - width / 2) ** 2)
+                h1 = math.sqrt((r_wall - r_roof) ** 2 - (r_wall - width / 2) ** 2)
                 # h_middle = round(height - r_roof - h1, 8)
                 h_middle = height - r_roof - h1
 
                 # ------------------------------------------------
-                if isna(r_wall_bottom):
+                if r_wall_bottom is None:
                     cross_section.add(None, width / 2)
                     cross_section.add(h_middle, width / 2)
 
@@ -226,7 +224,7 @@ class CrossSectionHolding(CrossSection):
             label (str): see :py:attr:`~__init__`
             height (float): see :py:attr:`~__init__`
             width (float): see :py:attr:`~__init__`
-            channel (Optional[float]): diameter of the dry weather channel
+            channel (Optional[float]): diameter of the dry weather channel (same unit as heght and width)
             bench (Optional[float]): bench (=Berme)
 
                 - ``''``: flache Berme
@@ -254,30 +252,31 @@ class CrossSectionHolding(CrossSection):
             see :doc:`show_case-kasten`
         """
         name = 'K'
-        cross_section = cls(label=label, width=width, height=height, **kwargs)
+        cross_section = cls(label=label, height=height, width=width, **kwargs)
 
-        if isna(channel):
-            channel = None
-        else:
+        # ------------------------------------------------
+        if channel is not None:
             channel = float(channel)
 
-        if isna(bench):
+        if bench is None:
             bench = ''
 
-        if isna(roof):
+        if roof is None:
             roof = ''
 
+        # ------------------------------------------------
         if channel or bench:
             name += '.'
             bench = str(bench).strip()
 
             if isinstance(channel, float):
-                name += '{:0.0f}'.format(channel)
+                name += f'{channel:0.0f}'
                 # diameter to radius
                 channel /= 2
                 cross_section.add(Circle(channel, x_m=channel))
 
             if isinstance(bench, str):
+                # '' | 'R' | 'H'
                 if bench != '45':
                     name += str(bench)
 
@@ -301,20 +300,26 @@ class CrossSectionHolding(CrossSection):
                     # cross_section.add(channel, width / 2)
                     # cross_section.add(channel + rounding, width / 2)
 
-                    if 1:
-                        cross_section.add(channel_end(channel, 45))
-                        cross_section.add(45, '°slope')
-                    else:
-                        cross_section.add(channel, channel)
-                    cross_section.add(channel + rounding, None)
-                    cross_section.add(5, '°slope')
+                    # --------either this
+                    cross_section.add(channel_end(channel, 45))
+                    cross_section.add(45, '°slope')
+                    # --------or this
+                    # cross_section.add(channel, channel)
+                    # --------
+                    if channel*2**(1/2) < (width/2):  # Eine Berme gibt es nur, wenn die Breite es zulässt.
+                        cross_section.add(channel + rounding, None)
+                        cross_section.add(5, '°slope')
+                    # else:
+                    #     print()
                     cross_section.add(None, width / 2)
 
         else:
             # ebene Sohle
             cross_section.add(0, width / 2)
 
+        # ------------------------------------------------
         if roof:
+            # '' | 'B' | 'K'
             name += '_' + str(roof)
 
             if roof == '':
@@ -323,7 +328,7 @@ class CrossSectionHolding(CrossSection):
 
             elif roof == 'B':
                 # Bogen-Decke
-                cross_section.add(height - width * (1 - cos(radians(30))), width / 2)
+                cross_section.add(height - width * (1 - math.cos(math.radians(30))), width / 2)
                 cross_section.add(Circle(width, x_m=height - width))
 
             elif roof == 'K':
@@ -334,6 +339,7 @@ class CrossSectionHolding(CrossSection):
             # gerade Decke
             cross_section.add(height, width / 2)
 
+        # ------------------------------------------------
         if cross_section.label is None or cross_section.label == '':
             cross_section.label = name
 
@@ -373,12 +379,13 @@ class CrossSectionHolding(CrossSection):
                 Kasten-Profile
         """
         infos = re.findall(r'(K)(\.?)(\d*)([RH]?)_?([BK]?)', label)  # _(\d+)x(\d+)
+        # 'Pr_K.30K' == 'Pr_K.30_K'
         if len(infos) == 1:
             infos = infos[0]
             _, _, channel, bench, roof = infos
 
             if channel != '':
-                channel = float(channel)
+                channel = float(channel) * 10  # cm to mm
             else:
                 channel = None
 
@@ -389,7 +396,7 @@ class CrossSectionHolding(CrossSection):
 
         # --------------------------------------
         else:
-            raise NotImplementedError('"{}" unknown !'.format(label))
+            raise NotImplementedError(f'"{label}" unknown !')
 
     @classmethod
     def from_point_cloud(cls, relative_coordinates, *args, **kwargs):
@@ -426,9 +433,9 @@ class CrossSectionHolding(CrossSection):
         """
         if isinstance(relative_coordinates, (list, tuple)):
             x, y = zip(*relative_coordinates)
-            x = array(x)
-            y = array(y)
-        elif isinstance(relative_coordinates, ndarray):
+            x = np.array(x)
+            y = np.array(y)
+        elif isinstance(relative_coordinates, np.ndarray):
             x = relative_coordinates[:, 0]
             y = relative_coordinates[:, 1]
         else:
@@ -440,12 +447,12 @@ class CrossSectionHolding(CrossSection):
         cross_section = cls(*args, **kwargs)
 
         # for the interpolation
-        y = append(y, [0])
-        x = append(x, [0])
+        y = np.append(y, [0])
+        x = np.append(x, [0])
 
         yi = sorted(set(y))
-        xi = interp(yi, y[:argmax(y) + 1], x[:argmax(y) + 1]) - \
-             interp(yi, y[::-1][:argmax(y[::-1]) + 1], x[::-1][:argmax(y[::-1]) + 1])
+        xi = np.interp(yi, y[:np.argmax(y) + 1], x[:np.argmax(y) + 1]) - \
+             np.interp(yi, y[::-1][:np.argmax(y[::-1]) + 1], x[::-1][:np.argmax(y[::-1]) + 1])
         xi /= 2
         for x, y in zip(yi, xi):
             if y == 0 and x in (0, height):
